@@ -2,6 +2,7 @@
 library("mlt")
 library("sandwich")
 set.seed(29)
+options(digits = 5)
 
 ### Nadja Klein
 dat <- data.frame(matrix(rnorm(300),ncol=3))
@@ -15,7 +16,7 @@ ctmm2 <- ctm(response = Bernstein_basis(y, order = 4, ui = "increasing"),
                               x2=Bernstein_basis(x2, order= 3)))
 ### fit model
 mltm2 <- mlt(ctmm2, data = dat, scale = TRUE)
-(p <- predict(mltm2, newdata = data.frame(x1=0, x2 = 0), q = mkgrid(mltm2, n = 10)[["y"]]))
+round(p <- predict(mltm2, newdata = data.frame(x1=0, x2 = 0), q = mkgrid(mltm2, n = 10)[["y"]]), 2)
 ### plot data
 plot(mltm2,newdata=expand.grid(x1=0:1, x2 = 0:1))
 
@@ -95,3 +96,12 @@ m <- ctm(as.basis(~ y, data = d, ui = matrix(c(0, 1), nr = 1), ci = 0),
 cf <- coef(mlt(m, data = d, fixed = c("x" = 1, "(Intercept)" = .5)))
 stopifnot(all.equal(cf[c("(Intercept)", "x")], 
                     c("(Intercept)" = .5, "x" = 1)))
+
+### by Balint Tamasi
+exact <- c(1, NA, NA)
+cleft <- c(NA, 2, -Inf)
+cright <- c(NA, Inf, 3)
+(resp <- R(exact, cleft = cleft, cright = cright))
+### was not the same
+(surv <- as.Surv(resp))
+
